import java.io.FileReader
import scala.util.parsing.combinator._

class ARTParser extends JavaTokenParsers {
  def ars: Parser[List[(String, List[String])]] = rep1sep(node, opt('\n'))

  def node: Parser[(String, List[String])] = idnum ~ ("->" | ":") ~ nodelist ^^ {case a ~ ("->" | ":") ~ b => (a, b)} |
          idnum ^^ {case a => (a, Nil)}

  def nodelist: Parser[List[String]] = rep1sep(idnum, ',')

  def idnum: Parser[String] = ident | wholeNumber // Number or Identifier
}

object ARTParser {
  def resultLengthOk(result: List[(String, List[String])]): Boolean = { // Checks if there are Nodes with a name thats to long
    def ok(s: String) = s.length < 3
    result.forall(x => ok(x._1) && x._2.forall(ok(_)))
  }
}

/**
 * Parses Abstract Rewrite System from File
 */
object ARTFileParser extends ARTParser {
  def apply(file: String) = {
    val reader = new FileReader(file)
    parseAll(ars, reader).get
  }
}

/**
 * Parses Abstract Rewrite System from String
 */
object ARTStringParser extends ARTParser {
  def apply(str: String) = {
    try {
      Some(parseAll(ars, str).get)
    } catch {
      case ex: RuntimeException => None
    }
  }
}