/**
 * Runs different Benchmarks
 */
object Benchmark {
  def main(args: Array[String]) {
    /*for(nodes <- 1 to 30)
      println(nodes + " " + Helper.nsToMs((for(i <- 1 to 100000) yield benchAllProperties(nodes)).reduceLeft(_+_)))*/
    val graph = DirectedGraph.getRandomGraph(10,1)
    val guiGraph = new GUIDirectedGraph(graph,1000,1000)
    println(LatexOutput.getLatexSource(guiGraph,"Testgraph"))
  }


  def runForceDirectedBenchmark() {
    for (numberOfNodes <- 1 to 40) {
      val l = for (i <- 1 to 50) yield {
        val graph = DirectedGraph.getRandomGraph(numberOfNodes, 1)
        val guiGraph = new GUIDirectedGraph(graph, 1000, 1000)
        val force = new ForceDirectedPlacement(guiGraph, 1000, 1000)
        Helper.getExecTime {for (i <- 1 to 1000) force.iterate()}
      }
      println(numberOfNodes + " " + l.foldLeft(0.toLong)(_ + _) / l.length + " ms")
    }
  }

  def benchAllProperties(size:Int) = {
    import concurrent.ops.future
    val graph = DirectedGraph.getRandomGraph(size,1)
    Helper.getExecTime {
      for(node <- graph) {
        node.isWN
        node.isUN
        node.isSN
        node.isWCR
        node.isCR
      }
    }
  }

  def testSN() {
    println((1 to 10000).forall {
      y =>
        val graph = DirectedGraph.getRandomGraph(10, 1)
        graph.forall(x => if (!x.isSN) graph.notSnNodes.contains(x) else !graph.notSnNodes.contains(x))
    })
  }

  var ll: Iterable[Node[GUIPayload]] = Nil

  def benchComponentSn() {
    println("Optimized")
    for (comp <- 1 to 1) {
      val l = for (i <- 1 to 100) yield {
        val graph = DirectedGraph.getRandomGraph(500, 1)
        Helper.getExecTime {ll = graph.notSnNodes}
      }
      println(comp + " " + l.sum / 1000000)
    }
  }

  def benchNormSn() {
    println("Not Optimized")
    for (comp <- 1 to 1) {
      val l = for (i <- 1 to 100) yield {
        val graph = DirectedGraph.getRandomGraph(500, 1)
        Helper.getExecTime {ll = for (i <- graph if (!i.isSN)) yield i}
      }
      println(comp + " " + l.sum / 1000000)
    }
  }

}