import java.lang.Math._
import PointPreamble._
import GUIPayloadPreamble._
import ProcessingWrapper._

/**
 * Created by IntelliJ IDEA.
 * User: Sebastian Stabinger
 * Date: May 11, 2010
 * Time: 3:21:21 AM
 */

class ForceDirectedPlacement(val guiGraph: GUIDirectedGraph, val w: Int, val l: Int) {
  val graph = guiGraph.graph
  val center = Point((w+XBORDER) / 2, (l+YBORDER) / 2)
  // Aktuelle "Temperatur" des Systems
  var t = 300
  // Temperature at which intersections are not allowed anymore
  var cutoff_temperature = 15
  // Anzahl an Versuchen wenn es noch Überschneidungen gibt
  var tries = 13
  // Fläche des Fames
  val area = w * l
  // Anzahl an Ecken im Graph
  def numberOfVertices = graph.nodes.hash.size max 10
  // Gewünschter Abstand zwischen den Ecken im gezeichneten Graph
  def k: Float = sqrt(area / numberOfVertices).toFloat / 2
  // Attraktorfunktion
  def fa(d: Double): Double = -(d * d) / k
  // Repulsorfunktion
  def fr(d: Double): Double = if (d > 2) (k * k) / d else 1000
  // Centerforce
  def fc(d: Double): Double = -d
  // Cooling function
  def cool() = t = ((t / 5) * 4) max 3

  // Calculates distance to the center
  def DCenter(pos: Point): Float = pos.distanceTo(Point(0, 0)).toFloat
  // Calculates distance between two nodes
  def D(n1: Node[GUIPayload], n2: Node[GUIPayload]): Double = n1.pos distanceTo n2.pos

  // Diplaces n1 with Amount disp regarding to position pos
  def dispNode(n1: Node[GUIPayload], pos: Point, disp: Double): Unit = {
    val alph = n1.pos angleTo pos
    n1.disp = n1.disp + ((disp * cos(alph)).toInt, (disp * sin(alph)).toInt)
  }
  // Diplaces n1 with Amount disp regarding to node n2
  def dispNode(n1: Node[GUIPayload], n2: Node[GUIPayload], disp: Double): Unit = dispNode(n1, n2.pos, disp)

  def calcRepulsiveForces() = {
    for (v <- graph) {
      for (u <- guiGraph.getNodesInArea(v.pos.x, v.pos.y, (2 * k).toInt) if (v != u)) {
        if (v.pos == u.pos) v.pos += 4 //Point(GetRandom(),GetRandom())
        dispNode(v, u, fr(D(v, u)))
      }
    }
  }

  def calcAttractiveForces() = {
    var visited: List[(Node[GUIPayload], Node[GUIPayload])] = Nil
    for (v <- graph; u <- v if (v != u && !(visited contains (v, u)))) {
      dispNode(v, u, fa(D(v, u)))
      dispNode(u, v, fa(D(u, v)))
      visited = (u, v) :: visited
    }
  }

  def calcCenterForces() = {
    def getDisplacement(nodePos: Point): Point = {
      val alph = nodePos angleTo center
      val disp = fc(DCenter(nodePos))
      Point((disp * cos(alph)).toInt, (disp * sin(alph)).toInt)
    }
    val displacementList = for (v <- graph) yield {
      getDisplacement(v.pos)
    }
    val averageDisplacement: Point = displacementList.reduceLeft(_ + _) / displacementList.size
    for (n <- graph) n.disp = n.disp + averageDisplacement
  }

  def limitDisplacement() = {
    for (n <- graph) {
      // Limit Displacement to current temperature
      n.disp = n.disp min (t, t)
      n.disp = n.disp max (-t, -t)
    }
  }

  def calculateNewPositions() = {
    for (n <- graph) {
      n.pos = n.pos + n.disp
      n.pos = n.pos max (XBORDER, YBORDER)
      n.pos = n.pos min (w, l)
      n.disp = (0, 0)
    }
  }

  // Ein Schritt des Optimierungsalgorithmus
  var iteration = 0

  def iterate() = {
    if (graph.nonEmpty) {
      //if(iteration % 2 == 0) println(LatexOutput.getGraphLatexOutput(guiGraph,"Iteration: "+iteration))
      //iteration += 1
      calcRepulsiveForces()
      calcAttractiveForces()
      calcCenterForces()
      limitDisplacement()
      calculateNewPositions()
      cool()
      if (t < cutoff_temperature && t > 3 && tries > 0 && guiGraph.gotIntersections) {
        t = 300
        tries -= 1
      }
    }
  }
}