import GUIPayloadPreamble._
//case class Gene(n:Node, p:(Int,Int))

object LatexOutput {
  def getGraphLatexOutput(graph: GUIDirectedGraph): String = {
    // Calculate scaling factors
    val boundingBox = graph.boundingBox
    def textsf(s:String) = "\\textsf{"+s+"}"

    def scalePoint(p: Point): (Float, Float) = {
      (p.x.toFloat / 70, p.y.toFloat / (-70))
    }
    "\\begin{tikzpicture}[scale=1.0]\n%nodes\n" +
            (for (i <- graph.graph.toList.sortWith(_.name < _.name)) yield {
              val (px, py) = scalePoint(i.pos)
              "\\node (" + i.name + ")  at (" + px + "," + py + ") {" + textsf(i.name) + "};"
            }).mkString("\n") +
            "\n%edges\n" +
            (for (i <- graph.graph.nodes if (i.connections != Nil)) yield {
              (i.connections map {
                x =>
                  "\\draw[->] (" + i.name + ") " +
                          (if (i == x) "edge [loop above]" else "--") +
                          " (" + x.name + ");"
              }).mkString("\n")
            }).mkString("\n") +
            "\n\\end{tikzpicture}\n"
  }

  def getPropertiesLatexOutput(graph: DirectedGraph):String = {
    def toStr(b:Boolean) = if(b) "\\checkmark" else "\\texttimes"
    "\\begin{tabular}{c|ccccc}\n" + " & SN & WN & CR & WCR & UN\\\\ \\hline\n" +
      (for(n <- graph.toList.sortWith(_.name < _.name)) yield {
        "\t\\textsf{"+n.name+"} & " + toStr(n.isSN) + " & " + toStr(n.isWN) + " & " +
                toStr(n.isCR) + " & " + toStr(n.isWCR) + " & " + toStr(n.isUN) + " \\\\"
      }).mkString("\n") +
    "\n\\end{tabular}"
  }

  def getOutput(graph: GUIDirectedGraph, caption: String = ""):String = {
    def getCaption = (if (caption != "") "\\caption{" + caption + "}\n" else "")
    "\\begin{figure}[htbp]\n"+
    "\\begin{minipage}{0.5\\linewidth}\n"+
    "\\centering\n"+
    getGraphLatexOutput(graph)+"\n"+
    "\\end{minipage}\n"+
    "\\begin{minipage}{0.5\\linewidth}\n"+
    "\\centering\n"+
    getPropertiesLatexOutput(graph.graph)+"\n"+
    "\\end{minipage}\n"+
    getCaption +
    "\\end{figure}\n"
  }

  def getLatexSource(graph:GUIDirectedGraph, caption:String = ""):String = {
    "\\documentclass[a4paper,10pt]{article}\n"+
    "\\usepackage[utf8x]{inputenc}\n"+
    "\\usepackage{textcomp} % needed for checkmark, texttimes\n"+
    "\\usepackage{tikz} % needed for textsf\n"+
    "\\usepackage{amssymb}\n"+
    "\\begin{document}"+getOutput(graph,caption)+"\\end{document}"
  }
}