import GeometricFunctions._
import GUIPayloadPreamble._
import PointPreamble._
import ProcessingWrapper._

/**
 * Created by IntelliJ IDEA.
 * User: Sebastian Stabinger
 * Date: Aug 14, 2010
 * Time: 11:55:42 PM
 */

/**
 * Implements certain basic geometric functions.
 */
object GeometricFunctions {
  /**Checkes if one has to go clockwise or counterclockwise to visit p3 from p0 and p1
   * @param p0, p1, p2 The three points to check
   * @return 0 if p2 lies on a line between p0 and p1
   * @return -1 if 
   */
  def ccw(p0: Point, p1: Point, p2: Point): Int = {
    val dx1 = p1.x - p0.x
    val dy1 = p1.y - p0.y
    val dx2 = p2.x - p0.x
    val dy2 = p2.y - p0.y
    if (dx1 * dy2 > dy1 * dx2) return -1
    if (dx1 * dy2 < dy1 * dx2) return 1
    if ((dx1 * dx2 < 0) || (dy1 * dy2 < 0)) return -1
    if ((dx1 * dx1 + dy1 * dy1) < (dx2 * dx2 + dy2 * dy2)) return 1
    return 0
  }
}

object Line {
  def zeroLine = new Line((0, 0), (0, 0))
  // Factory Method
  def apply(p1: Point, p2: Point) = new Line(p1, p2)

  def apply(n1: Node[GUIPayload], n2: Node[GUIPayload]) = new Line(n1.pos, n2.pos)

  def apply(pair: (Node[GUIPayload], Node[GUIPayload])) = new Line(pair._1.pos, pair._2.pos)
}

class Line(val p1: Point, val p2: Point) {
  /**Checks if two lines intersects
   * @return true if the line intersects with that
   * @return false if the line doesnt intersect with that or if the two lines share a common endpoint
   */
  def intersects(that: Line): Boolean = {
    if (p1 == that.p1 || p1 == that.p2 || p2 == that.p1 || p2 == that.p2) false
    else
      ((ccw(p1, p2, that.p1) * ccw(p1, p2, that.p2)) <= 0) &&
              ((ccw(that.p1, that.p2, p1) * ccw(that.p1, that.p2, p2)) <= 0)
  }

  def intersects(n1: Node[GUIPayload], n2: Node[GUIPayload]): Boolean = {
    intersects(Line(n1.pos, n2.pos))
  }

  // http://mathworld.wolfram.com/Circle-LineIntersection.html
  // TODO: Bessere Implementierung
  def intersectsCircle(center: Point, radius: Int): Boolean = {
    intersects(Line((center.x - radius, center.y), (center.x + radius, center.y))) ||
            intersects(Line((center.x, center.y - radius), (center.x, center.y + radius)))
  }

  // Returns true if the line intersects with the loop indicating a "selfedge"
  def intersectsSelfEdge(center: Point, radius: Int): Boolean = {
    // Calculate 2 Lines inside the loop
    val verticalLine = Line((center.x, center.y - radius), (center.x, center.y - radius - 25))
    val horizontalLine = Line((center.x - 12, center.y - radius - 10), (center.x + 12, center.y - radius - 10))
    intersects(verticalLine) || intersects(horizontalLine)
  }

  override def toString = p1.toString + p2.toString
}