import processing.core.PApplet
import processing.core.{PConstants => pC}

/**
 * Created by IntelliJ IDEA.
 * User: Sebastian Stabinger
 * Date: Sep 6, 2010
 * Time: 2:58:11 AM
 */

object MessageSystem {
  def apply(p: PApplet): MessageSystem = {
    val mS = new MessageSystem(p)
    new Thread(mS).start()
    mS
  }
}

class MessageSystem(val p: PApplet) extends Runnable {
  var messageList = List[Message]()
  val maxLines = 5

  case class Message(text: String, duration: Int, color: Int, size: Int)

  def run = {
    while (true) {
      messageList = for (m <- messageList if (m.duration > 1)) yield {
        Message(m.text, m.duration - 1, m.color, m.size)
      }
      Thread.sleep(1000)
    }
  }

  /**
   * Shows a message on screen
   */
  def displayMessage(text: String, color: Int = ProcessingWrapper.WHITE, size: Int = 18, yOffset: Int = 0) = {
    val x = p.width / 2
    val y = p.height - (size / 2) - 10 - yOffset

    p.textSize(size)
    p.textAlign(pC.CENTER, pC.CENTER)
    p.fill(color)

    p.text(text, x, y)

    p.textSize(ProcessingWrapper.standardTextSize)
  }

  def displayMessage(m: Message, yOffset: Int) {
    displayMessage(m.text, m.color, m.size, yOffset)
  }

  def addMessage(text: String, duration: Int = 10, color: Int = ProcessingWrapper.WHITE, size: Int = 15) {
    messageList = Message(text, duration, color, size) :: messageList
  }

  def clear() = messageList = Nil

  /**
   * When called will draw all Messages on the PApplet
   */
  def draw() = {
    var offset = 0
    messageList = messageList take maxLines
    messageList foreach {
      m =>
        displayMessage(m, offset)
        offset = offset + m.size + 4
    }
  }
}