import PointPreamble._
import ProcessingWrapper._
import processing.core.PImage
import processing.core.PConstants._

/**
 * Factoryobject for convenient use of Constructor
 * instead of new PButton(p1,p2)(   { .... } )
 * PButton(p1,p2)   { .... }
 */
object PButton {
  def apply(center: Point, width: Int, height: Int)(f: => Unit) = new PButton(center, width, height)(f)
}

/**
 * Implements basic functionality of a button
 */
class PButton(val center: Point, val width: Int, val height: Int)(f: => Unit) {
  var visible = true
  val p1 = center - (width, height) / 2 // Point in the upper left corner
  val p2 = center + (width, height) / 2 // Point in the lower right corner

  def check(p: Point) {
    if (visible && (p between (p1, p2))) f
  }

  def draw() {
    if (visible) {
      p.noFill()
      p.rect(p1.x, p1.y, width, height)
    }
  }
}

/**
 * Factoryobject for convenient use of Constructor
 */
object PImageButton {
  def apply(im: PImage, center: Point)(f: => Unit) = new PImageButton(im, center)(f)
}

/**
 * Implements functionality of a button containing an image
 */
class PImageButton(val im: PImage, center: Point)(f: => Unit) extends PButton(center, im.width, im.height)(f) {
  override def draw() {
    if (visible) {
      p.imageMode(CENTER)
      p.image(im, center.x, center.y)
    }
  }
}
