import scala.math._

/**
 * Created by IntelliJ IDEA.
 * User: Sebastian Stabinger
 * Date: May 13, 2010
 * Time: 3:27:15 PM
 */
/**
 * ul: Upper Left
 * lr: Lower Right
 */
case class Box(ul: Point, lr: Point)

case class Point(x: Int, y: Int) {
  def distanceTo(t: Point): Double = sqrt(pow((x - t.x), 2d) + pow((y - t.y), 2d))

  def angleTo(t: Point): Double = {
    val tmp = this - t
    val sign = if (tmp.y > 0) 1 else -1
    sign * acos(tmp.x / sqrt(pow(tmp.x, 2) + pow(tmp.y, 2)))
  }

  def +(t: Point) = Point(x + t.x, y + t.y)

  def +(i: Int) = Point(x + i, y + i)

  def -(t: Point) = new Point(x - t.x, y - t.y)

  def -(i: Int) = new Point(x - i, y - i)

  def /(t: Point) = new Point(x / t.x, y / t.y)

  def /(i: Int) = new Point(x / i, y / i)

  def *(t: Point) = new Point(x * t.x, y * t.y)

  def *(i: Int) = new Point(x * i, y * i)

  def <(t: Point) = (x < t.x) && (y < t.y)

  def >(t: Point) = (x > t.x) && (y > t.y)

  // Returns a Point which holds the max-x/y-Value of the two points
  def max(t: Point) = new Point(x max t.x, y max t.y)

  // Returns a Point which holds the min-x/y-Value of the two points
  def min(t: Point) = new Point(x min t.x, y min t.y)

  // Returns true if "this" lies in a box built by p1 and p2
  def between(p1: Point, p2: Point): Boolean = (this > p1) && (this < p2)
}

object PointPreamble {
  implicit def tupleWrapper(t: (Int, Int)) = new Point(t._1, t._2)
}
