import processing.core.PApplet
import scala.math._
import PointPreamble._
import processing.core.{PConstants => pC}

/**
 * Created by IntelliJ IDEA.
 * User: Sebastian Stabinger
 * Date: May 10, 2010
 * Time: 7:00:02 PM
 */

/**
 * Wraps a PApplet in an object so it can be used in every class after import and adds some additional functionality
 */
object ProcessingWrapper {
  var p: PApplet = null

  val standardTextSize = 20

  def lazyTest[A](f: => A) = {
    require(p != null, "Tried to access a lazy val before assigning PApplet a Value")
    f
  }

  // Using def to force lazy evaluation since p isn't defined at creation of object
  lazy val RED = lazyTest {p.color(255, 0, 0)}

  lazy val GREEN = lazyTest {p.color(0, 255, 0)}

  lazy val BLUE = lazyTest {p.color(0, 0, 255)}

  lazy val YELLOW = lazyTest {p.color(255, 255, 0)}

  lazy val WHITE = lazyTest {p.color(255, 255, 255)}

  lazy val BLACK = lazyTest {p.color(0)}

  lazy val GAMEMODECOLOR = lazyTest {p.color(255)}

  lazy val EDITMODECOLOR = lazyTest {p.color(255)}

  lazy val LINECOLOR = lazyTest {p.color(0)}

  lazy val TEXTCOLOR = lazyTest {p.color(0)}

  lazy val MARKED_PROPERTY_COLOR = lazyTest {p.color(255, 187, 0)}

  lazy val NODE_FILL_COLOR = lazyTest {p.color(255)}

  lazy val NODE_BORDER_COLOR = lazyTest {p.color(0)}

  lazy val NODE_MARKED_FILL_COLOR = lazyTest {p.color(230, 230, 255)}

  lazy val NODE_TEXT_COLOR = lazyTest {p.color(0)}

  var NODE_NF_TEXT_COLOR = 0

  lazy val MARKED_PATH_COLOR = lazyTest {RED}

  lazy val TIME_COUNTER_COLOR = lazyTest {p.color(255, 187, 0)}

  lazy val MISTAKE_COUNTER_COLOR = lazyTest {RED}

  lazy val RIGHT_MARK_COLOR = lazyTest {p.color(184, 255, 186)}

  lazy val WRONG_MARK_COLOR = lazyTest {p.color(255, 184, 184)}

  lazy val PARSE_PROMPT_COLOR = lazyTest {p.color(255, 187, 0, 200)}

  lazy val PARSE_PROMPT_TEXT_COLOR = lazyTest {p.color(0)}

  lazy val tickImg = lazyTest {p.loadImage("tick.png")}

  lazy val crossImg = lazyTest {p.loadImage("cross.png")}

  lazy val questionImg = lazyTest {p.loadImage("question.png")}

  lazy val scissorsImg = lazyTest {p.loadImage("scissors.png")}

  lazy val astImg = lazyTest {p.loadImage("asterisk.png")}

  lazy val acceptImg = lazyTest {p.loadImage("accept.png")}

  lazy val declineImg = lazyTest {p.loadImage("discard.png")}

  lazy val gameHelpImage = lazyTest {p.loadImage("gamehelp.png")}
  
  lazy val editHelpImage = lazyTest {p.loadImage("edithelp.png")}

  val XSIZE = 600

  val YSIZE = 500

  val XBORDER = 15
  val YBORDER = 130

  def setParent(parent: PApplet) {
    this.p = parent
  }

  def line(p1: Point, p2: Point, color: Int): Unit = {p.stroke(color); p.line(p1.x, p1.y, p2.x, p2.y)}

  def line(l: Line, color: Int = WHITE): Unit = line(l.p1, l.p2, color)

  def arrow(x1: Int, y1: Int, x2: Int, y2: Int, margin: Int) = {
    val alph = ((x1, y1) angleTo (x2, y2)).toFloat
    val xn2 = (x2 + (margin * cos(alph))).toFloat
    val yn2 = (y2 + (margin * sin(alph))).toFloat
    val xn1 = (x1 - (margin * cos(alph))).toFloat
    val yn1 = (y1 - (margin * sin(alph))).toFloat
    p.line(xn1, yn1, xn2, yn2)

    def drawArrowHead() {
      p.pushMatrix()
      p.translate(xn2, yn2)
      val a = math.atan2(xn1 - xn2, yn2 - yn1)
      p.rotate(a.toFloat)
      p.line(0, 0, -6, -15)
      p.line(0, 0, 6, -15)
      p.popMatrix()
    }

    drawArrowHead()
  }

  def arrow(p1: Point, p2: Point, margin: Int): Unit = arrow(p1.x, p1.y, p2.x, p2.y, margin: Int)

  def arrow(l: Line, margin: Int): Unit = arrow(l.p1, l.p2, margin)

  def circle(center: Point, radius: Int) = p.ellipse(center.x, center.y, radius * 2, radius * 2)

  /**
   * Draws a rect with upper-left-point p1 and lower-right-point p2
   */
  def rect(p1: Point, p2: Point) {
    val wh = p2 - p1
    p.rect(p1.x, p1.y, wh.x, wh.y)
  }

  /**
   * Shows a message on screen
   */
  def message(text: String, color: Int = WHITE, size: Int = 18) {
    val x = p.width / 2
    val y = p.height - (size / 2)

    p.textSize(size)
    p.textAlign(pC.CENTER, pC.CENTER)
    p.fill(color)

    p.text(text, x, y)

    p.textSize(standardTextSize)
  }
}
